# meds/bots/__init__.py
#
#

""" bot base class. """

from meds.errors import ENOTIMPLEMENTED
from meds.scheduler import Scheduler
from meds.launcher import Launcher
from meds.utils.name import sname
from meds.object import Object
from meds.event import Event

import logging
import select
import sys
import os

class Bot(Scheduler, Launcher):

    """ base class for bot implementations. """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._resume = Object()
        self.poll = select.epoll()
        self.type = type(self)
        self.channels = []

    def resume(self): pass

    def loop(self):
        self._status = "run"
        while self._status:
            for event in self.events(): self.put(event)

    def add(self, f):
        logging.info("# add %s" % str(f))
        try: fd = f.fileno()
        except: fd = f
        self.poll.register(fd)
        return fd

    def event(self):
        e = self._queue.get()
        return e

    def events(self):
        for (fd, events) in self.poll.poll(-1): yield self.event()

    def cmnd(self, txt):
        event = Event()
        event._bot = self
        event.origin = "user@bot"
        event.txt = txt
        self.put(event)
        return event
         
    def reply(self, event, txt):
        self._nr += 1
        if type(txt) in [list, tuple]: self.say(event.channel, ",".join(txt))
        else: self.say(event.channel, txt)

    def out(self, txt):
        """ lowlevel output function. """
        raise ENOTIMPLEMENTED

    def say(self, channel, txt):
        """ output text to channel. """
        self.out(txt)

    def announce(self, txt):
        """ announce text. """
        for channel in self.channels: self.say(channel, txt)

    def join(self, channel):
        """ join a channel. """
        pass

    def joinall(self):
        """ join all channels. """
        for channel in self.channels: self.join(channel)
