# cli.py
#
#

""" CLI bot """

from meds.utils.name import sname
from meds.log import YELLOW, ENDC
from meds.errors import ENOFUNC
from meds.event import Event
from meds.bots import Bot
from meds.cfg import cfg

import readline
import logging
import _thread
import select
import time
import tty
import sys
import os

class CLI(Bot):

    """ console line interface bot. """

    def dispatch(self, event):
        from meds.kernel import kernel
        thrs = kernel.dispatch(event)
        if thrs: kernel.waiter(thrs)

    def start(self, *args, **kwargs):
        """ run command line command. """
        if not self.channels: self.channels.append("#meds")
        fd = self.add(sys.stdin)
        self._resume.fd = sys.stdin.fileno()
        if not cfg.main.shell:
            if not cfg.main.args: return
            self.launch(super().start)
            event = self.cmnd(" ".join(cfg.main.args))
            event.wait()
            return
        self.launch(self.loop, name="%s.loop" % sname(self))
        super().start()

    def stop(self):
        """ stop CLI bot. """
        from meds.kernel import kernel
        kernel.fleet.CLI.remove(self)
        self.poll.close()
        self.kill("CLI")
     
    def resume(self): self.poll = select.epoll.fromfd(self._resume.fd)

    def events(self):
        for (fd, events) in self.poll.poll(-1):
            event = Event()
            event._bot = self
            event.txt = str(os.read(fd, 512), "utf-8")
            event.txt = event.txt.rstrip()
            event.origin = "root@shell"
            yield event

    def out(self, txt):
        """ output text to stdout. """
        sys.stdout.write(txt)
        sys.stdout.write("\n")
        sys.stdout.flush()

    def prompt(self):
        """ return bot prompt. """
        if cfg.main.colors: txt = "%s>%s " % (YELLOW, ENDC)
        else: txt = "> "
        sys.stdout.write(txt)
        sys.stdout.flush()
