# xmpp.py
#
#

""" xmpp bot. """

from meds.utils.trace import get_strace, get_exception
from meds.utils.misc import stripped
from meds.errors import EPASSWORD
from meds.kernel import kernel
from meds.event import Event
from meds.bots import Bot
from meds.cfg import cfg

import threading
import logging
import getpass
import queue
import time
import ssl
import os

class XMPP(Bot):

    """ XMPP chat bot. """

    def __init__(self):
        super().__init__()
        self._queue = queue.Queue()
        self._connected = threading.Event()
        self.channels = []

    def makeclient(self, jid, password):
        """ create sleekxmpp client part. """
        import sleekxmpp
        self.jid = jid
        self.port = 5222
        self.client = sleekxmpp.clientxmpp.ClientXMPP(jid, password)
        #self.add(self.client)
        self.register("session_start", self.session_start)
        self.register("message", self.messaged)
        self.register("iq", self.iqed)
        self.register('disconnected', self.disconnected)
        self.register('connected', self.connected)
        self.register('presence_available', self.presenced)
        self.register('presence_dnd', self.presenced)
        self.register('presence_xa', self.presenced)
        self.register('presence_chat', self.presenced)
        self.register('presence_away', self.presenced)
        self.register('presence_unavailable', self.presenced)
        self.register('presence_subscribe', self.presenced)
        self.register('presence_subscribed', self.presenced)
        self.register('presence_unsubscribe', self.presenced)
        self.register('presence_unsubscribed', self.presenced)
        self.register('groupchat_message', self.messaged)
        self.register('groupchat_presence', self.presenced)
        self.register('groupchat_subject', self.presenced)
        self.register('failed_auth', self.failedauth)
        self.client.exception = self.exception
        self.client.use_signals()

    def start(self):
        config = kernel.last("cfg", "xmpp")
        if not config: config = cfg.xmpp ; config.save()
        try: f = open("sleekpass", "r") ; pw = f.read() ; f.close()
        except: logging.error("missing sleekpass file") ; return
        self.makeclient(config.user, pw)
        self.update(config)
        self.connecting()
        self.launch(self.core)
        self.launch(super().start, name="XMPP.start")

    def stop(self):
        """ stop XMPP bot. """
        if "client" in self and self.client: self.launch(self.client.disconnect)
        self._queue.put(None)
        super().stop()
        self.kill("XMPP")

    def event(self):
        """ block on _queue until data arrives. """
        return self._queue.get()

    def connecting(self):
        """ connect to XMPP server. """
        logging.info("! connect XMPP %s" % self.user)
        self.client.reconnect_max_attempts = 1
        if cfg.main.openfire:
            logging.warn("# openfire enabled")
            #self.client.ssl_version = ssl.PROTOCOL_SSLv3
            self.client.connect((self.server, cfg.xmpp.port), use_ssl=True, reattempt=True)
        else: self.client.connect(reattempt=False)

    def register(self, key, value):
        """ register a handler. """
        logging.info("! register XMPP %s" % key)
        self.client.add_event_handler(key, value)

    def core(self):
        """ core XMPP client loop. """
        self.client.process(block=True)

    def say(self, jid, txt):
        """ output text to jid. """
        txt = str(txt)
        self.client.send_message(jid, txt)
        logging.info("> %s %s" % (stripped(jid), txt))

    def announce(self, txt):
        """ announce text on all registered channels. """
        for channel in self.channels:
            try: self.say(channel, txt)
            except: logging.error(get_exception())

    def out(self, txt):
        """ lowlevel output function. """
        self.client.send_raw(txt)

    def session_start(self, data):
        """ session is started callback. """
        self.client.send_presence()
        self.ready()
        logging.info("! session XMPP %s" % self.user)
         
    def exception(self, data):
        """ log an exception. """
        self._error = data
        logging.error("^ error XMPP %s"% str(data))

    def failedauth(self, data):
        """ log a failed authentication attempt. """
        self._error = data
        logging.error("^ EAUTH %s" % str(data))

    def failure(self, data):
        """ log a failure. """
        self._error = data
        logging.error("^ error XMPP %s" % str(data))

    def disconnected(self, data):
        """ disconnect callback. """
        time.sleep(5.0)
        self._connected.clear()
        self._status = "disconnect"
        logging.warn("^ disconnect %s" % self.user)

    def connected(self, data):
        """ connect callback. """ 
        self._status = "connected"
        self._connected.set()
        logging.warn("# connected %s:%s" % (self.server, self.port))

    def iqed(self, data):
        """ iq callback. """ 
        print(data)
        logging.warn("# Iq %s:%s" % (self.server, self.port))

    def pinged(self, event): print(event)

    def messaged(self, data):
        """ handle incoming messages. """
        logging.debug("< %s" % data)
        m = Event()
        m._bot = self
        m.update(data)
        if m.type == "error": logging.error("^ %s" % m.error) ; return
        m.cc = ""
        m["from"] = str(m["from"])
        if self.user in m["from"]: logging.info("! ignore XMPP %s %s" % (m.type, m["from"])) ; return
        m.origin = m["from"]
        m.channel = m.origin
        m.to = m.origin
        m.element = "message"
        m.txt = m["body"]
        if '<delay xmlns="urn:xmpp:delay"' in str(data):
            logging.info("! ignore XMPP %s" % m.origin)
            return
        logging.info("< %s %s" % (stripped(m.origin), m.txt))
        self.put(m)

    def presenced(self, data):
        """ handle presence notices. """
        logging.debug("< %s" % data)
        o = Event()
        o._bot = self
        o.update(data)
        o["from"] = str(o["from"])
        o.origin = o["from"]
        if "txt" not in o: o.txt = ""
        o.element = "presence"
        username = o.origin
        if o.type == 'subscribe':
            pres = Event({'to': o["from"], 'type': 'subscribed'})
            self.client.send_presence(pres)
            pres = Event({'to': o["from"], 'type': 'subscribe'})
            self.client.send_presence(pres)
        elif o.type == "unavailable" and username != self.jid and username in self.channels: self.channels.remove(username)
        elif o.origin != self.user and username != self.jid and username not in self.channels: self.channels.append(username)
        o.no_dispatch = True
        o.txt = o.type
        logging.info("< %s %s" % (stripped(o.origin), o.type))
        self.put(o)
