""" launch threads.  """

from meds.utils.name import name as thrname
from meds.utils.trace import get_exception
from meds.errors import EREBOOT
from meds.object import Object

import threading
import _thread
import logging
import queue
import time

class Task(threading.Thread):

    def __init__(self, *args, **kwargs):
        super().__init__(*args)
        self.type = "Task"
        self._last = time.time()
        self._name = ""
        self._queue = queue.Queue()
        self._ready = threading.Event()
        self._result = None
        self._start = time.time()
        self._status = "start"
        self.args = args
        self.kwargs = kwargs
        self.setDaemon(True)
        self.once = True

    def __iter__(self):
        for x in dir(self): yield x

    def isSet(self): return self._ready.isSet()

    def ready(self): self._ready.set() ; logging.debug("! ready %s" % repr(self))

    def clear(self): self._ready.clear() ; logging.debug("! clear %s" % repr(self))

    def wait(self, sec=180.0): logging.debug("! wait %s %s" % (repr(self), self._ready.wait(sec)))

    def join(self, *args, **kwargs):
        super().join(*args, **kwargs)
        return self._result

    def put(self, func, *args):
        self._queue.put_nowait((func, args))
        return self

    def run(self):
        from meds.event import Event
        self._status = "working"
        while self._status != "stopped":
            event = None
            _func, args = self._queue.get()
            if not self._status or not _func: break
            self._name = self.name or thrname(_func)
            self.setName(self._name)
            self._begin = time.time()
            self._result =  _func(*args)
            self._last = time.time()
            if len(args) and type(args[0]) == type(Event()):
                logging.warn("# %s.%s %.6fs" % (_func.__module__, thrname(_func), self._last - self._begin))
                args[0].done = True
                args[0].ready()
            if self.once: break
        self.status = "stopped"
        return self._result

    def exit(self):
        self._status = "stopped"
        self.put(None, None, None)

class Launcher(Object):

    cc = "!"
    default = ""
    
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._status = "init"
        self.tasks = []
        self._nr = 0
        
    def waiter(self, thrs, timeout=60.0):
        result = []
        for thr in thrs:
            res = thr.join(timeout)
            result.append(res)
        return result

    def launch(self, *args, **kwargs):
        self._nr += 1
        t = Task(**kwargs)
        t.name = kwargs.get("name") or thrname(args[0])
        t.start()
        t.once = True
        t.put(*args)
        return t

    def kill(self, name):
        thrs = []
        for thr in self.running(name):
            if "exit" in dir(thr): thr.exit()
            if "stop" in dir(thr): thr.stop()
            if "cancel" in dir(thr): thr.cancel() 
            logging.warn("# killed %s" % thrname(thr))
            thrs.append(thr)
        return thrs

    def running(self, name=""): 
        for thr in threading.enumerate():
            if str(thr).startswith("<_"): continue
            if name and name not in str(thr): continue
            yield thr
