""" rest interface. """

from meds.utils.trace import get_exception
from meds.launcher import Launcher
from meds.storage import Storage
from meds.object import Object
from meds.cfg import cfg

from meds import __version__

import logging
import time

from http.server import HTTPServer, BaseHTTPRequestHandler

class REST(HTTPServer, Object):

    allow_reuse_address = True
    daemon_thread = True

    def __init__(self, *args, **kwargs):
        HTTPServer.__init__(self, *args, **kwargs)
        Object.__init__(self)
        self.host = args[0]
        self._status = "start"
        self._last = time.time()
        self._start = time.time()

    def exit(self):
        logging.warn("# end rest")
        self._status = ""
        time.sleep(0.2)
        self.shutdown()

    def start(self): 
        logging.warn("# start rest http://%s:%s" % self.host)
        self._status = "ok"
        self.ready()
        self.serve_forever()

    def request(self):
        self._last = time.time()

    def error(self, request, addr):
        ex = get_exception()
        logging.warn('# error rest %s %s' % (addr, ex))

class RESTHandler(BaseHTTPRequestHandler):

    def setup(self):
        BaseHTTPRequestHandler.setup(self)
        self._ip = self.client_address[0]
        self._size = 0

    def write_header(self, type='text/plain'):
        self.send_response(200)
        self.send_header('Content-type', '%s; charset=%s ' % (type, "utf-8"))
        self.send_header('Server', __version__)
        self.end_headers()

    def do_GET(self):
        filename = ""
        for fn in store.all():
            if fn.endswith(self.path): filename = fn ; break
        try: f = open(fn, "r") ; txt = f.read() ; f.close()
        except (TypeError, FileNotFoundError): self.send_response(404) ; self.end_headers() ; return
        txt = txt.replace("\\n", "\n")
        txt = txt.replace("\\t", "\t")
        self.write_header()
        self.wfile.write(bytes(txt, "utf-8"))
        self.wfile.flush()

    def log(self, code): logging.warn('# log rest %s code %s path %s' % (self.address_string(), code, self.path))

launcher = Launcher()
store = Storage()
objs = Object()
port = 10102

def init(event):
    objs.rest = REST((cfg.rest.hostname, int(cfg.main.port or cfg.rest.port)), RESTHandler)
    launcher.launch(objs.rest.start)

def stop(event):
    if "server" in main: main.server.exit()
    launcher.kill("REST")
