# meds/rss.py
#
#

""" rss module. """

from meds.utils.url import get_url, strip_html, get_feed
from meds.utils.tijd import to_time, file_time
from meds.utils.misc import matching
from meds.utils.join import sj
from meds.launcher import Launcher
from meds.storage import storage
from meds.clock import Repeater
from meds.errors import ENODATE
from meds.object import Object
from meds.kernel import kernel
from meds.event import Event
from meds.objs import objs
from meds.cfg import cfg

import logging
import time

class RSS(Object):

    def start(self):
        logging.warn("# start rss")
        config  = storage.last("cfg", "rss")
        if config: cfg.rss.update(config)
        repeater = Repeater(600, self.fetcher)
        return kernel.launch(repeater.start)

    def stop(self): kernel.kill("RSS")
       
    def fetcher(self):
        thrs = []
        for fn in storage.all("rss"):
            thr = kernel.launch(self.fetch, fn)
            thrs.append(thr)
        result = kernel.waiter(thrs)
        if result: logging.warn("# fetched %s" % (sj(*[str(x) for x in result if x]) or 0))
        objs.seen.sync()

    def fetch(self, fn):
        nr = 0
        obj = Object().load(fn)
        if obj.get("deleted", None): return
        if not obj.get("rss", None): return 
        for o in get_feed(obj.rss):
            if o.link in objs.seen.list: continue
            objs.seen.list.append(o.link)
            o.prefix = "feeds"
            o.services = "rss"
            if "published" in o:
                try: date = file_time(to_time(o.published)) ; o.save(date)
                except ENODATE as ex: logging.warn("EDATE %s" % str(ex))
            else: o.save()
            kernel.announce(self.display(o))
            nr += 1
        return nr 

    def display(self, obj):
        result = ""
        for key in cfg.rss.display_list:
            data = obj.get(key, None)
            if data: result += "%s - " % strip_html(data.rstrip())
        for check in cfg.rss.descriptions:
            if check in obj.link:
                summary = obj.get("summary", None)
                if summary: result += "%s - " % summary
        if result: return result[:-3].rstrip()
