# store.py
#
#

""" read files from disk. """

from meds.utils.selector import selector, wanted, notwanted
from meds.utils.getters import matching
from meds.utils.tijd import fn_time
from meds.object import Object
from meds.errors import ELOAD, ENODIR
from meds.utils.join import j
from meds.event import Event
from meds.cfg import cfg

import logging
import glob
import os

class Storage(Object):

    def scan(self, path, *args, **kwargs):
        p = Event(**kwargs)
        if args: prefix = args[0]
        else: prefix = ""
        if not path.endswith(os.sep): path += os.sep
        if prefix and os.path.isdir(path + prefix + os.sep): path += prefix + os.sep
        for root, dirs, files in os.walk(path, topdown=True):
            for fn in files:
                fnn = j(root, fn)
                timed = fn_time(fnn)
                if timed and p.start and timed < p.start: continue
                if timed and p.end and timed > p.end: continue
                yield fnn

    def all(self, *args, **kwargs):
        p = cfg.main.workdir
        return sorted(self.scan(p, *args, **kwargs), key=lambda x: fn_time(x))

    def prefixed(self, *args, **kwargs):
        p = cfg.main.workdir
        if args and not os.path.isdir(j(p, args[0])): raise ENODIR(j(p, args[0]))
        return sorted(self.scan(p, *args, **kwargs), key=lambda x: fn_time(x))

    def selected(self, event):
        for fn in self.all(*event.fields, **event.boundaries):
            try: obj = Object().load(fn)
            except: logging.warn("fail %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            #if not obj.get(event.prefix, None): continue
            if event.fields and not selector(obj, event.fields): continue
            if event.notwant and notwanted(obj, event.notwant): continue
            if event.want and not wanted(obj, event.want): continue
            yield obj

    def since(self, *args, **kwargs):
        e = Event(**kwargs)
        e.start = args[1]
        for fn in self.prefixed(*args, **e):
            try: obj = Object().load(fn)
            except: logging.warn("fail %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            if len(args) > 1 and obj.get(args[0]) != args[1]: continue
            yield obj
 
    def first(self, *args, **kwargs):
        for fn in self.prefixed(*args, **kwargs):
            try: obj = Object().load(fn)
            except: logging.warn("fail %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            if len(args) > 1 and obj.get(args[0]) != args[1]: continue
            return obj

    def last(self, *args, **kwargs):
        e = Event(**kwargs)
        for fn in self.prefixed(*args, **kwargs)[::-1]:
            try: obj = Object().load(fn)
            except: logging.warn("fail %s" % fn) ; continue
            if "deleted" in obj and obj.deleted: continue
            if len(args) > 1 and obj.get(args[0]) != args[1]: continue
            if "txt" in obj and e.exclude in obj.txt: continue
            return obj

storage = Storage()
