# udp.py
#
#

""" relay txt through a udp port listener. """

from meds.utils.name import name
from meds.kernel import kernel
from meds.object import Object
from meds.cfg import cfg

import logging
import socket
import time

class UDP(Object):

    """ UDP class to echo txt through the bot, use the meds-udp program to send. """

    def __init__(self):
        Object.__init__(self)
        self._status = "start"
        self._start = time.time()
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self._sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
        self._sock.setblocking(1)

    def start(self):
        """ start the UDP server. """
        logging.warn("# start udp %s:%s" % (cfg.udp.host, cfg.udp.port))
        self._sock.bind((cfg.udp.host, cfg.udp.port))
        self.ready()
        self._status = "ok"
        while self._status:
            (txt, addr) = self._sock.recvfrom(64000)
            if not self._status: break
            data = str(txt.rstrip(), "utf-8")
            if not data: break
            self.output(data, addr)
        logging.warn("# stop udp %s:%s" % (cfg.udp.host, cfg.udp.port))

    def exit(self):
        """ shutdown the UDP server. """
        logging.warn("# exit %s" % name(self))
        self._status = ""
        self._sock.settimeout(1.0)
        self._sock.sendto(bytes("bla", "utf-8"), (cfg.udp.host, cfg.udp.port))

    def output(self, txt, addr=None):
        """ output to all bots on fleet. """
        (passwd, text) = txt.split(" ", 1)
        text = text.replace("\00", "")
        if passwd == cfg.udp.password:
            kernel.announce(text)

