# meds/utils/obj.py
#
#

""" object accessors. """

from meds.utils.tijd import to_date, to_time, elapsed, fn_time, rtime
from meds.utils.file import check_permissions
from meds.errors import ENODATE
from meds.utils.join import j

import time
import os

dirmask = 0o700
filemask = 0o600

def fnn(obj):
    fn = obj.f_code.co_filename
    return fn.split(os.sep)[-1][:-3]

def urled(obj):
    from meds.cfg import cfg
    if "_path" in obj: fn = os.sep.join(obj._path.split(os.sep)[-2:])
    else: fn = ""
    return "http://%s:%s/%s" % (cfg.rest.hostname, cfg.main.port, fn)

def root(obj):
    from meds.cfg import cfg
    path = cfg.main.workdir
    path = os.path.abspath(path)
    check_permissions(path)
    return path

def path(obj):
    if "workdir" in obj: p = obj.workdir
    else: p = root(obj)
    if "prefix" in obj: p = j(p, obj.prefix)
    return os.path.abspath(p) 

def days(obj):
    t1 = time.time()
    t2 = timed(obj)
    if t2:
        time_diff = float(t1 - t2)
        return elapsed(time_diff)

def dated(obj):
    val = ""
    if "Date" in obj: val = obj.Date
    elif "date" in obj: val = obj.date
    elif "published" in obj: val = obj.published
    elif "created" in obj: val = obj.created
    elif "added" in obj: val = obj.added
    elif "saved" in obj: val = obj.saved
    elif "timed" in obj: val = obj.timed
    elif "_state" in obj: val = obj._state.saved
    if not val: raise ENODATE
    return val

def timed(obj):
    t = to_date(dated(obj))
    if t: t = to_time(t)
    if not t and "_path" in obj: t = fn_time(obj._path)
    if not t: t = fn_time(rtime())
    return t

def look(obj, name):
    l = []
    for key in obj.keys():
        if name in key: l.append(key)
    return l

def match(obj, txt):
    for key in obj.keys():
        if txt not in key: continue
        return obj[key]

def matching(obj, keys):
    for key in keys:
        if not getattr(obj, key, None): return False
    return True

def slice(obj, keys=[]):
    from meds.object import Object
    o = Object()
    if not obj: return o
    if not keys: keys = obj.keys()
    for key in keys:
        if key.startswith("_"): continue
        try: val = obj[key]
        except KeyError: continue
        try: val.keys() ; o[key] = slice(val)
        except: o[key] = val
    return o

